# -*- coding: utf-8 -*-

import datetime

from django.db.models import Q

from kinder.core.direct.models import Direct
from kinder.core.direct.models import DRS


def get_children_last_direct_ids(mo_id, directs_filter=Q()):
    """Возвращает отфильтрованные последние направления детей.

    Используется для расчета показателей 32, 32.1, 32.2.

    Последнее выданное направление, если у ребенка их несколько в периоде:
        - Направления в статусе "Направлен в ДОУ", "Заключение договора",
    "Желает изменить ДОУ", "Не явился", "Отказано", "Зачислен".
        - дата создания направления: [01.01.2019 - текущая дата],
    т.е текущий календарный год.

    :param: mo_id: ID МО для заявлений которого выбираются направления.
    :param: directs_filter: Фильтр статусов заявлений среди последних.

    :return: QuerySet
    """

    today_ = datetime.date.today()
    filter_ = Q(
        status__code__in=(
            DRS.DOGOVOR, DRS.REGISTER, DRS.NEED_CHANGE,
            DRS.REJECT, DRS.REFUSED, DRS.ACCEPT
        ),
        date__range=(datetime.date(today_.year, 1, 1), today_),
    )

    # фильтр для выбора последних заявлений детей
    unique_directs = Direct.objects.filter(
        declaration__mo__id=mo_id
    ).filter(
        filter_
    ).order_by(
        'declaration__children', '-date'
    ).distinct(
        'declaration__children'
    ).values_list(
        'id', flat=True
    )

    # фильтруются уже среди последних заявлений детей
    return Direct.objects.filter(
        directs_filter,
        id__in=unique_directs,
    ).values_list('id', flat=True)


def get_32_filter():
    """Фильтр для показателя 32.

    Направления в статусе "Зачислен".
    """
    return Q(status__code__in=(DRS.ACCEPT,))


def get_321_filter():
    """Фильтр для показателя 32.1.

    Направления в статусе "Направлен в ДОУ", "Заключение договора",
    "Желает изменить ДОУ", "Не явился", "Отказано", "Зачислен".
    """
    return Q(status__code__in=(
        DRS.DOGOVOR, DRS.REGISTER, DRS.NEED_CHANGE,
        DRS.REJECT, DRS.REFUSED, DRS.ACCEPT
    ))


def get_322_filter():
    """Фильтр для показателя 32.1.

    Направления в статусе "Не явился", "Желает изменить ДОО".
    """
    return Q(status__code__in=(DRS.REJECT, DRS.NEED_CHANGE))
